/*
ジュリア殺害の日の様子
*/
open util/ordering[Time]
enum Time{夜,深夜}

////////////////////////////////////
//////
//////登場主体に関する記述
//////
////////////////////////////////////
abstract sig Character {
	enter:  this -> Room one -> Time,
}
//動物
abstract sig Animal extends Character {}
one sig ヒヒ,チーター,小さい有毒生物 extends Animal {}
//人間
abstract sig Person extends Character {
	Kill : this lone -> lone ( Person - this ) -> Method -> Time,
	lock:  this -> Room -> Time,
}
one sig  ヘレン,ジュリア,ロイロット extends Person {}
////////////////////////////////////
//////
//////殺人方法に関する記述
//////
////////////////////////////////////
fact {
all p: Person | all t: Time { 殺害[Person,p,t] => no p.Kill} 
}
abstract sig Method {
}
one sig 直接, 薬物, 隙間凶器, 小生物 extends Method {}
--殺害方法の列挙
pred 殺害[p : Person  , q : Person , t:Time]{直接[p, q, t] || 薬物[p, q, t] || 隙間[p, q, t]  || 生物[p, q, t]}
//直接
pred 直接[p : Person  , q : Person , t:Time]{ one p.Kill.t.直接.q && no p.Kill.Time.(Method - 直接) && Together1[p, q, t]}
//薬物
pred 薬物[p : Person  , q : Person , t:Time]{ one p.Kill.t.薬物.q && no p.Kill.Time.(Method - 薬物)&& Together1[p, q, (t.prevs)]}
//隙間凶器
pred 隙間[p : Person  , q : Person , t:Time]{ one p.Kill.t.隙間凶器.q && no p.Kill.Time.(Method - 隙間凶器)&& not Together1[p, q, t]  && Passable[(~(p.enter.t)).p,(~(q.enter.t)).q]}
//小さい生き物
pred 襲う[a : Animal, t: Time ]{one p:Person | one p.Kill.t.小生物 && no p.Kill.Time.(Method - 小生物) && one q:( Person - p ) | Together1[a,q,t]}
pred 生物[p : Person  , q : Person , t:Time]{ one p.Kill.t.小生物.q && no p.Kill.Time.(Method - 小生物)&& not Together1[p,q, t] && one a:Animal | not Together1[p,a, t] && not Together2[p,a, t] &&  襲う[a,t]}

//事故または自殺
//pred 事故＆自殺[p : Person  , t:Time]{ all q:( Person - p ) | no q.Kill.t.Method.p || ( 薬物[p,p,t] || 襲う[Animal,t] ) }

--殺人方法による結果の定義
//凶器、隙間凶器使うと外傷ができる、遅効性薬物使うと薬物反応がでる
pred 外傷[p: Person ] { one t:Time | 直接[ Person, p, t ]  || 隙間[ Person, p, t] }
pred 毒物[p: Person ] { one t:Time | 薬物[ Person, p, t ] }
--殺人による結果
pred Dead[p: Person, t: Time]{殺害[Person,p,t]  }
pred Alive[p: Person, t: Time]{not Dead[p,t]}
////////////////////////////////////
//////
//////建物・所在に関する記述
//////
////////////////////////////////////
--建物に関する定義
abstract sig Room {
	owner : set Person,
	pass: set Room
}
one sig ヘレン部屋,ジュリア部屋, ロイロット部屋, 金庫,外 extends Room{}
fact {
//部屋はownerが内側からしかlockできない。金庫はownerがlockできる
	all r: ( Room - 金庫 ) | all p: Person | all t: Time { some p.lock.t.r => p in r.owner && some p.enter.t.r}
	all r: 金庫 | all p: Person | all t: Time { some p.lock.t.r => p in r.owner}
//部屋が、ある人にlockされてたら、その人以外は入れない。
	all disj p, q: Person  | all r: (Room - 金庫) | all t: Time { some p.lock.t.r => no q.enter.t.r }
//金庫に人は入れない
	all p: Person {not p -> 金庫 in  p.enter.Time}
//金庫に入っているものはロイロットが鍵をかけている限り出れない。開けたら動き回れる
	all t: Time {no ロイロット.enter.t.ロイロット部屋 => some ロイロット.lock.t.金庫 }
	all t: Time {one ロイロット.lock.t.金庫 => lone Character.enter.t.金庫 }
	//all t: Time {no ロイロット.lock.t.金庫  => one Character.enter.t.(金庫 +  ジュリア部屋 + ロイロット部屋)  }	
//隙間がある部屋間

	ロイロット部屋.pass=( ジュリア部屋  + ロイロット部屋)
	ジュリア部屋.pass=(  ジュリア部屋  + ロイロット部屋 )
	all r:( Room - ロイロット部屋 - ジュリア部屋) {no r.pass}
} 

--所在に関する定義
pred Together1[ p : Character  , q : Character , t:Time ]{ ~(p.enter.t).p = ~(q.enter.t).q}
pred Together2[ p : Character  , q : Character , t:Time ]{ ~(p.enter.t).p.owner = ~(q.enter.t).q.owner}
pred Passable[r:Room, s:Room]{ s in r.pass}
pred Alone[ p : Character , t:Time ]{ not Together1[p, Person,t] }

--動物が動き回れる範囲
fact{
all a:Animal { a -> 外 in a.enter.夜 => a -> 外 in a.enter.深夜}
all a:Animal { not a -> 外 in a.enter.夜 => not a -> 外 in a.enter.深夜}
all a:Animal {
	lone a.enter.Time.金庫
	lone a.enter.Time.ジュリア部屋
	lone a.enter.Time.ロイロット部屋
	lone a.enter.Time.外
	all r:( Room - 金庫 - ジュリア部屋 - ロイロット部屋 - 外 ) { no a.enter.Time.r}
	}
}

////////////////////////////////////
////////////////////////////////////
//////
//////場面に関するFACT
//////
////////////////////////////////////
////////////////////////////////////

//場面に関する情報
//場所に関する情報（言及あり）
fact {
ジュリア部屋.owner=ジュリア
ヘレン部屋.owner=ヘレン
ロイロット部屋.owner=ロイロット
金庫.owner=ロイロット
ジュリア.lock.深夜=ジュリア -> ジュリア部屋
ヘレン.lock.深夜=ヘレン -> ヘレン部屋
Together1[ヘレン, ジュリア, 夜]
ヘレン.enter.夜=ヘレン -> ヘレン部屋
ジュリア.enter.夜=ジュリア-> ヘレン部屋
ヘレン.enter.深夜=ヘレン -> ヘレン部屋
ジュリア.enter.深夜=ジュリア -> ジュリア部屋
ロイロット.enter.深夜=ロイロット -> ロイロット部屋
ヒヒ.enter.夜=ヒヒ -> 外
チーター.enter.夜=チーター -> 外
}
//状態に関する情報（言及あり）
fact {
Dead[ジュリア,深夜]
Alive[ヘレン,夜]
Alive[ジュリア,夜]
Alive[ヘレン,深夜]
Alive[ロイロット,夜]
Alive[ロイロット,深夜]
not 外傷[ジュリア]
not 毒物[ジュリア]

}

//ヘレンが正直者だとした
pred honest[p: Person]{ no p.Kill}
fact {
//ヘレンはhonestである。
	//honest[ヘレン]
}

//これより下、解に影響がない範囲で与えた計算上都合上の束縛条件
//場面からの推測（外部知識でもある）
fact {
not Together1[ヘレン, Animal, 夜]
ロイロット.enter.夜=ロイロット -> ロイロット部屋
no 外.owner
no Character.enter.夜.ジュリア部屋
}

pred show {
#Person.lock.夜<1
#Person.lock.深夜<3
}
run show
